##
# DBM::Deep Test
##
use strict;
use Test::More tests => 16;
use Test::Exception;
use t::common qw( new_fh );

use_ok( 'DBM::Deep' );

my ($fh, $filename) = new_fh();
my $db = DBM::Deep->new( $filename );

$db->{foo} = '1234';
$db->{foo} = '2345';

my $size = -s $filename;
$db->{foo} = '3456';
cmp_ok( $size, '==', -s $filename, "A second overwrite doesn't change size" );

$size = -s $filename;
delete $db->{foo};
cmp_ok( $size, '==', -s $filename, "Deleted space isn't released" );

$size = -s $filename;
$db->{bar} = '2345';
cmp_ok( $size, '==', -s $filename, "Added a new key after a delete reuses space" );

$db->{baz} = {};
$size = -s $filename;

delete $db->{baz};
$db->{baz} = {};

cmp_ok( $size, '==', -s $filename, "delete and rewrite reuses space" );

$db->{baz} = {};
$size = -s $filename;

$db->{baz} = {};

cmp_ok( $size, '==', -s $filename, "delete and rewrite reuses space" );

my $x = { foo => 'bar' };
$db->{floober} = $x;

delete $db->{floober};

ok( !exists $x->{foo}, "Deleting floober makes \$x empty (exists)" );
is( $x->{foo}, undef, "Deleting floober makes \$x empty (read)" );
is( delete $x->{foo}, undef, "Deleting floober makes \$x empty (delete)" );

eval { $x->{foo} = 'bar'; };
is( $@, "Cannot write to a deleted spot in DBM::Deep.\n", "Exception thrown when writing" );

cmp_ok( scalar( keys %$x ), '==', 0, "Keys returns nothing after deletion" );

$db->{buzzer} = { foo => 'baz' };

ok( !exists $x->{foo}, "Even after the space has been reused, \$x is still empty" );
is( $x->{foo}, undef, "Even after the space has been reused, \$x is still empty" );
is( delete $x->{foo}, undef, "Even after the space has been reused, \$x is still empty" );

eval { $x->{foo} = 'bar'; };
is( $@, "Cannot write to a deleted spot in DBM::Deep.\n", "Exception thrown when writing" );

cmp_ok( scalar( keys %$x ), '==', 0, "Keys returns nothing after space reuse" );
